@@ -35,6 +35,7 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
+#include "types.h"
 #include "distcc.h"
 #include "trace.h"
 #include "access.h"
@@ -96,6 +97,8 @@ int dcc_parse_mask(const char *spec,
 
 /**
  * Check whether a client ought to be allowed.
+ *
+ * @returns 0 for allowed, or EXIT_ACCESS_DENIED.
  **/
 int dcc_check_address(in_addr_t client,
                       in_addr_t value,
