@@ -28,20 +28,34 @@
  *
  * Functions for understanding and manipulating argument vectors.
  *
- * @todo We don't need to run the full argument scanner on the
- * server, only something simple to recognize input and output files.
- * That would perhaps make the function simpler, and also mean that if
- * argument recognizer bugs are fixed in the future, they only need to
- * be fixed on the client, not on the server.
+ * @todo We don't need to run the full argument scanner on the server, only
+ * something simple to recognize input and output files.  That would perhaps
+ * make the function simpler, and also mean that if argument recognizer bugs
+ * are fixed in the future, they only need to be fixed on the client, not on
+ * the server.  An even better solution is to have the client tell the server
+ * where to put the input and output files.
  *
- * @todo Detect arguments which ask the assembler to produce a
- * listing in addition to an object file.  These must be run locally.
- * This may be complex, because it will look like -Wa,-alh or
- * something.  We need to see -Wa and then recursive
+ * @todo Detect arguments which ask the assembler to produce a listing in
+ * addition to an object file.  These must be run locally.  This may be
+ * complex, because it will look like -Wa,-alh or something.  We need to see
+ * -Wa and then recursively examine the sub-arguments.
  *
- * @todo Perhaps make the argument parser driven by a data table.
- * (Would that actually be clearer?)  Perhaps use regexps to recognize
- * strings.
+ * @todo Perhaps make the argument parser driven by a data table.  (Would that
+ * actually be clearer?)  Perhaps use regexps to recognize strings.
+ *
+ * @todo We could also detect options like "-x cpp-output" or "-x
+ * assembler-with-cpp", because they should override language detection based
+ * on extension.  I haven't seen anyone use them yet though.  In fact, since
+ * we don't assemble remotely it is moot for the only reported case, the
+ * Darwin C library.
+ *
+ * @todo Perhaps assume that assembly code will not use both #include and
+ * .include, and therefore if we preprocess locally we can distribute the
+ * compilation?
+ *
+ * @todo Apparently some compilers are unhappy about writing to a file that's
+ * not called .o.  gcc doesn't seem to have this problem.  This might be
+ * problematic if the server needs to assign the temporary filename.
  **/
 
 
@@ -50,7 +64,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <assert.h>
 #include <string.h>
 #include <fcntl.h>
 #include <errno.h>
@@ -111,9 +124,7 @@ static int dcc_argv_append(char *argv[], char *toadd)
  * results differently.  So it does not directly cause any actions, but only
  * updates fields in @p jobinfo.
  *
- * @retval 0 if it's ok to distribute this compilation.
- *
- * @retval -1 if it would be unsafe to distribute this compilation.
+ * @returns 0 if it's ok to distribute this compilation, or an error code.
  *
  * @todo Fix this to use standard return codes.
  **/
@@ -123,7 +134,6 @@ int dcc_scan_args(char *argv[], char **input_file, char **output_file,
     int seen_opt_c = 0, seen_opt_s = 0;
     int i;
     char *a;
-    char *argstr;
     int ret;
 
      /* allow for -o foo.o */
@@ -131,9 +141,7 @@ int dcc_scan_args(char *argv[], char **input_file, char **output_file,
         return ret;
     argv = *ret_newargv;
 
-    argstr = dcc_argv_tostr(argv);
-    rs_trace("scanning arguments: %s", argstr);
-    free(argstr);
+    dcc_trace_argv("scanning arguments", argv);
 
     /* Things like "distcc -c hello.c" with an implied compiler are
      * handled earlier on by inserting a compiler name.  At this
@@ -149,7 +157,7 @@ int dcc_scan_args(char *argv[], char **input_file, char **output_file,
         if (a[0] == '-') {
             if (!strcmp(a, "-E")) {
                 rs_trace("-E call for cpp must be local");
-                return -1;
+                return EXIT_DISTCC_FAILED;
             } else if (!strcmp(a, "-MD") || !strcmp(a, "-MMD") || 
                        !strcmp(a, "-MG") || !strcmp(a, "-MP")) {
                 /* -MD does *not* imply -E.  It will locally produce
@@ -167,22 +175,16 @@ int dcc_scan_args(char *argv[], char **input_file, char **output_file,
                     not the compiler.  There would be no point trying
                     to distribute it even if we could. */
                 rs_trace("%s implies -E (maybe) and must be local", a);
-                return -1;
+                return EXIT_DISTCC_FAILED;
             } else if (!strcmp(a, "-S")) {
                 seen_opt_s = 1;
             } else if (!strcmp(a, "-fprofile-arcs")
                        || !strcmp(a, "-ftest-coverage")) {
                 rs_log_info("compiler will emit profile info; must be local");
-                return -1;
+                return EXIT_DISTCC_FAILED;
             } else if (!strcmp(a, "-x")) {
-                /* TODO: We could also detect options like "-x
-                 * cpp-output" or "-x assembler-with-cpp", because they
-                 * should override language detection based on
-                 * extension.  I haven't seen anyone use them yet
-                 * though. */
-
                 rs_log_info("gcc's -x handling is complex; running locally");
-                return -1;
+                return EXIT_DISTCC_FAILED;
             } else if (!strcmp(a, "-c")) {
                 seen_opt_c = 1;
             } else if (!strcmp(a, "-o")) {
@@ -198,7 +200,7 @@ int dcc_scan_args(char *argv[], char **input_file, char **output_file,
                 rs_trace("found input file \"%s\"", a);
                 if (*input_file) {
                     rs_log_info("do we have two inputs?  i give up");
-                    return -1;
+                    return EXIT_DISTCC_FAILED;
                 }
                 *input_file = a;
             } else if (str_endswith(".o", a)) {
@@ -206,7 +208,7 @@ int dcc_scan_args(char *argv[], char **input_file, char **output_file,
                 rs_trace("found object/output file \"%s\"", a);
                 if (*output_file) {
                     rs_log_info("called for link?  i give up");
-                    return -1;
+                    return EXIT_DISTCC_FAILED;
                 }
                 *output_file = a;
             }
@@ -219,12 +221,12 @@ int dcc_scan_args(char *argv[], char **input_file, char **output_file,
 
     if (!seen_opt_c && !seen_opt_s) {
         rs_log_info("compiler apparently called not for compile");
-        return -1;
+        return EXIT_DISTCC_FAILED;
     }
 
     if (!*input_file) {
         rs_log_info("no visible input file");
-        return -1;
+        return EXIT_DISTCC_FAILED;
     }
 
     if (!*output_file) {
@@ -242,13 +244,13 @@ int dcc_scan_args(char *argv[], char **input_file, char **output_file,
          * preprocessing" rather than "stop after compilation." */
         if (seen_opt_s) {
             if (dcc_output_from_source(*input_file, ".s", &ofile))
-                return -1; 
+                return EXIT_DISTCC_FAILED; 
         } else if (seen_opt_c) {
             if (dcc_output_from_source(*input_file, ".o", &ofile))
-                return -1;
+                return EXIT_DISTCC_FAILED;
         } else {
             rs_log_crit("this can't be happening(%d)!", __LINE__);
-            return -1;
+            return EXIT_DISTCC_FAILED;
         }
         rs_log_info("no visible output file, going to add \"-o %s\" at end",
                       ofile);
@@ -265,7 +267,7 @@ int dcc_scan_args(char *argv[], char **input_file, char **output_file,
          * so we just always run it locally.  Hopefully this is a
          * pretty rare case. */
         rs_log_info("output to stdout?  running locally");
-        return -1;
+        return EXIT_DISTCC_FAILED;
     }
 
     return 0;
@@ -291,14 +293,11 @@ int dcc_shallowcopy_argv(char **from, /*@-nullstate@*/ char ***out, int delta)
     /*@null@*/ char **b;
     int l, i;
 
-    assert(out != NULL);
-    assert(from != NULL);
-
     l = dcc_argv_len(from);
     b = malloc((l+1+delta) * (sizeof from[0]));
     if (b == NULL) {
         rs_log_error("failed to allocate copy of argv");
-        exit(EXIT_FAILURE);
+        exit(EXIT_OUT_OF_MEMORY);
     }
     for (i = 0; i < l; i++) {
         b[i] = from[i];
@@ -322,14 +321,11 @@ int dcc_deepcopy_argv(char **from, /*@-nullstate@*/ char ***out)
 
     l = dcc_argv_len(from);
     
-    assert(out != NULL);
-    assert(from != NULL);
-
     l = dcc_argv_len(from);
     b = malloc((l+1) * (sizeof from[0]));
     if (b == NULL) {
         rs_log_error("failed to allocate copy of argv");
-        exit(EXIT_FAILURE);
+        exit(EXIT_OUT_OF_MEMORY);
     }
 
     for (i = 0; i < l; i++) 
@@ -355,7 +351,7 @@ int dcc_set_action_opt(char **a, const char *new_c)
             *a = strdup(new_c);
             if (*a == NULL) {
                 rs_log_error("strdup failed");
-                exit(EXIT_FAILURE);
+                exit(EXIT_OUT_OF_MEMORY);
             }
             gotone = 1;
             /* keep going; it's not impossible they wrote "gcc -c -c
@@ -364,7 +360,7 @@ int dcc_set_action_opt(char **a, const char *new_c)
 
     if (!gotone) {
         rs_log_error("failed to find -c or -S");
-        return -1;
+        return EXIT_DISTCC_FAILED;
     } else {
         return 0;
     }
@@ -395,7 +391,7 @@ int dcc_set_output(char **a, char *ofname)
         if (0==strcmp(a[i], "-o")  &&  a[i+1] != NULL) {
             rs_trace("changed output from \"%s\" to \"%s\"", a[i+1], ofname);
             a[i+1] = ofname;
-            rs_trace("command after: %s", dcc_argv_tostr(a));
+            dcc_trace_argv("command after", a);
             return 0;
         }
     
@@ -404,7 +400,7 @@ int dcc_set_output(char **a, char *ofname)
      * these commands. */
 
     rs_log_error("failed to find \"-o\"");
-    return -1;
+    return EXIT_DISTCC_FAILED;
 }
 
 
@@ -419,22 +415,17 @@ int dcc_set_output(char **a, char *ofname)
 int dcc_set_input(char **a, char *ifname)
 {
     int i;
-    char *astr;
-
-    astr = dcc_argv_tostr(a);
-    rs_trace("command before: %s", astr);
-    free(astr);
 
     for (i =0; a[i]; i++)
         if (dcc_is_source(a[i])) {
             rs_trace("changed input from \"%s\" to \"%s\"", a[i], ifname);
             a[i] = ifname;
-            rs_trace("command after: %s", dcc_argv_tostr(a));
+            dcc_trace_argv("command after", a);
             return 0;
         }
     
     rs_log_error("failed to find input file");
-    return -1;
+    return EXIT_DISTCC_FAILED;
 }
 
 
