@@ -215,8 +215,7 @@ int dcc_x_file(int ofd, const char *fname, const char *token,
 /**
  * Receive a file stream from the network into a local file.  
  *
- * This is only used for receipt into a regular file.  The file is
- * always removed before creation.
+ * This is only used for receipt into a regular file.
  *
  * @param filename local filename to create.  
  **/
@@ -224,13 +223,38 @@ int dcc_r_file(int ifd, const char *filename, size_t len)
 {
     int ofd;
     int ret, close_ret;
+    struct stat s;
 
-    if (unlink(filename) && errno != ENOENT) {
-        rs_log_error("failed to remove %s: %s", filename, strerror(errno));
-        return EXIT_IO_ERROR;
+    /* This is meant to behave similarly to the output routines in bfd/cache.c
+     * in gnu binutils, because makefiles or configure scripts may depend on
+     * it for edge cases.
+     *
+     * We try to remove the output file first, if its size is not 0.  That
+     * should make the newly created file be owned by the current user; it
+     * might also help in the dangerous case of some other process still
+     * reading from the file.
+     *
+     * I guess checking for size 0 helps avoid removing /dev/null when running
+     * as root.
+     *
+     * However, failure to remove the file does not cause a warning; we may
+     * not have write permission on the directory.  The main case of this is
+     * writing to /dev/null.
+     */
+
+    if (stat(filename, &s) == 0) {
+        if (s.st_size != 0) {
+            if (unlink(filename) && errno != ENOENT) {
+                rs_trace("failed to remove %s: %s", filename, strerror(errno));
+                /* continue */
+            }
+        }
+    } else {
+        rs_trace("stat %s failed: %s", filename, strerror(errno));
+        /* continue */
     }
 
-    ofd = open(filename, O_TRUNC|O_WRONLY|O_CREAT|O_EXCL|O_BINARY, 0666);
+    ofd = open(filename, O_TRUNC|O_WRONLY|O_CREAT|O_BINARY, 0666);
     if (ofd == -1) {
         rs_log_error("failed to create %s: %s", filename, strerror(errno));
         return EXIT_IO_ERROR;
