@@ -32,7 +32,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <assert.h>
 #include <string.h>
 #include <fcntl.h>
 #include <errno.h>
@@ -42,10 +41,11 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
+#include "types.h"
 #include "distcc.h"
 #include "trace.h"
 #include "io.h"
-#include "opt.h"
+#include "dopt.h"
 #include "exitcode.h"
 #include "setuid.h"
 #include "access.h"
@@ -64,6 +64,9 @@ int opt_daemon_mode = 0;
 int opt_inetd_mode = 0;
 int opt_no_fifo = 0;
 
+/** If non-NULL, listen on only this address. **/
+char *opt_listen_addr = NULL;
+
 struct dcc_allow_list *opt_allowed = NULL;
 
 /**
@@ -75,6 +78,12 @@ int opt_no_detach = 0;
 
 int opt_log_stderr = 0;
 
+/**
+ * Daemon exits after this many seconds.  Intended mainly for testing, to make
+ * sure daemons don't persist for too long.
+ */
+int opt_lifetime = 0;
+
 const char *arg_pid_file = NULL;
 const char *arg_log_file = NULL;
 
@@ -89,6 +98,8 @@ const struct poptOption options[] = {
     { "daemon", 0,       POPT_ARG_NONE, &opt_daemon_mode, 0, 0, 0 },
     { "help", 0,         POPT_ARG_NONE, 0, '?', 0, 0 },
     { "inetd", 0,        POPT_ARG_NONE, &opt_inetd_mode, 0, 0, 0 },
+    { "lifetime", 0,     POPT_ARG_INT, &opt_lifetime, 0, 0, 0 },
+    { "listen", 0,       POPT_ARG_STRING, &opt_listen_addr, 0, 0, 0 },
     { "log-file", 0,     POPT_ARG_STRING, &arg_log_file, 0, 0, 0 },
     { "log-stderr", 0,   POPT_ARG_NONE, &opt_log_stderr, 0, 0, 0 },
     { "nice", 'N',       POPT_ARG_INT, &arg_nice_inc,  'N', 0, 0 },
@@ -116,16 +127,16 @@ static void distccd_show_usage(void)
 "    --help                     explain usage and exit\n"
 "    --version                  show version and exit\n"
 "    -p, --port PORT            TCP port to listen on\n"
+"    --listen ADDRESS           IP address to listen on\n"
 "    -P, --pid-file FILE        save daemon process id to file\n"
 "    -N, --nice LEVEL           lower priority, 20=most nice\n"
 "    --user USER                if run by root, change to this persona\n"
 "    -a, --allow IP[/MASK]      client address access control\n"
 "  Debug and trace:\n"
 "    --verbose                  include debug messages in log\n"
-"    --no-fork                  single process only (for debugging ONLY)\n"
 "    --no-detach                don't detach from parent (for daemontools, etc)\n"
 "    --log-file=FILE            send messages here instead of syslog\n"
-"    --log-stderr               send messages to stderr (for debugging)\n"
+"    --log-stderr               send messages to stderr\n"
 "  Mode of operation:\n"
 "    --inetd                    serve client connected to stdin\n"
 "    --daemon                   bind and listen on socket\n"
