@@ -1,9 +1,8 @@
-/* -*- c-file-style: "java"; indent-tabs-mode: nil -*-
+/* -*- c-file-style: "java"; indent-tabs-mode: nil; fill-column: 78; -*-
  * 
  * distcc -- A simple distributed compiler system
- * $Header: /data/cvs/distcc/src/dparent.c,v 1.41 2003/01/27 14:01:40 mbp Exp $ 
  *
- * Copyright (C) 2002 by Martin Pool <mbp@samba.org>
+ * Copyright (C) 2002, 2003 by Martin Pool <mbp@samba.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -67,24 +66,20 @@
 #include "trace.h"
 #include "io.h"
 #include "util.h"
-#include "opt.h"
+#include "dopt.h"
 #include "exec.h"
 #include "tempfile.h"
 #include "srvnet.h"
+#include "types.h"
 
 #ifndef WAIT_ANY
 #  define WAIT_ANY (-1)
 #endif
 
 static int dcc_start_child(int listen_fd, int);
-static volatile int parent_exit_signal = 0;
 static void dcc_parent_loop(int listen_fd) NORETURN;
-static void dcc_parent_signalled(int);
-static void dcc_catch_signals(void);
-static void dcc_parent_terminate(void) NORETURN;
 static int dcc_serve_connection(int, int acc_fd);
 static void dcc_detach(void);
-static void dcc_remove_pid(void);
 static void dcc_save_pid(pid_t);
 
 static int nkids = 0;
@@ -98,9 +93,11 @@ static int nkids = 0;
 int dcc_standalone_server(void)
 {
     int listen_fd;
+    int n_cpus;
+    int ret;
 
-    if ((listen_fd = open_socket_in(arg_port)) == -1)
-        return EXIT_BIND_FAILED;
+    if ((ret = dcc_socket_listen(arg_port, &listen_fd, opt_listen_addr)) != 0)
+        return ret;
 
     set_cloexec_flag(listen_fd, 1);
 
@@ -108,6 +105,9 @@ int dcc_standalone_server(void)
            "distccd (version %s, built %s %s) listening on port %d",
            PACKAGE_VERSION, __DATE__, __TIME__, arg_port);
 
+    if (dcc_ncpus(&n_cpus) == 0) 
+        rs_log_info("%d CPU%s online", n_cpus, n_cpus == 1 ? "" : "s");
+
     dcc_catch_signals();
 
     if (!opt_no_detach) {
@@ -123,77 +123,6 @@ int dcc_standalone_server(void)
 }
 
 
-/**
- * Catch and remember various fatal signals, which will cause the
- * parent and all children to exit.
- **/
-static void dcc_parent_signalled(int whichsig)
-{
-    parent_exit_signal = whichsig;
-}
-
-
-/**
- * Signal handler for SIGCHLD.  Does nothing, but calling into this
- * ought to make us break out of waiting for a connection.
- **/
-static void dcc_child_exited(int UNUSED(whichsig))
-{
-}
-
-
-/**
- * Signals are caught only in the parent at the moment, because we
- * want to be able to kill off all children and log a message when
- * exiting.  This is perhaps not the best thing to do -- it might be
- * better to simply let ourselves be terminated, and for children to
- * go away when they're finished their current work.  If the admin is
- * really impatient, they can kill all the children anyhow.
- **/
-static void dcc_catch_signals(void)
-{
-    struct sigaction act_catch, act_exited;
-
-    memset(&act_catch, 0, sizeof act_catch);
-    act_catch.sa_handler = dcc_parent_signalled;
-
-    memset(&act_exited, 0, sizeof act_exited);
-    act_exited.sa_handler = dcc_child_exited;
-    act_exited.sa_flags = SA_NOCLDSTOP;
-
-    if (sigaction(SIGTERM, &act_catch, NULL)
-        || sigaction(SIGINT, &act_catch, NULL)
-        || sigaction(SIGHUP, &act_catch, NULL)
-        || sigaction(SIGQUIT, &act_catch, NULL)
-        || sigaction(SIGCHLD, &act_exited, NULL)) {
-        rs_log_error("failed to install signal handlers: %s",
-                     strerror(errno));
-    }
-}
-
-
-/**
- * Ignore hangup signal.
- *
- * This is only used in detached mode to make sure the daemon does not
- * quit when whoever started it closes their terminal.  In nondetached
- * mode, the signal is logged and causes an exit as normal.
- **/
-static void dcc_ignore_sighup(void)
-{
-    struct sigaction act_ignore;
-
-    memset(&act_ignore, 0, sizeof act_ignore);
-    act_ignore.sa_handler = SIG_IGN;
-
-    if (sigaction(SIGHUP, &act_ignore, NULL)) {
-        rs_log_error("failed to install signal handler: %s",
-                     strerror(errno));
-    } else {
-        rs_trace("ignoring SIGHUP");
-    }
-}
-
 
 /**
  * @sa dcc_wait_child(), which is used by a process that wants to do a blocking
@@ -245,11 +174,15 @@ static void dcc_reap_kids(void)
  **/
 static void dcc_parent_loop(int listen_fd)
 {
-    while (parent_exit_signal == 0) {
+    while (1) {
         int acc_fd;
+        struct sockaddr cli_addr;
+        socklen_t cli_len;
 
         rs_log_info("waiting to accept connection");
-        acc_fd = accept(listen_fd, NULL, 0);
+
+        cli_len = sizeof cli_addr;
+        acc_fd = accept(listen_fd, &cli_addr, &cli_len);
         if (acc_fd == -1 && errno == EINTR) {
             /* just go ahead and check for children */
         }  else if (acc_fd == -1) {
@@ -265,11 +198,8 @@ static void dcc_parent_loop(int listen_fd)
             dcc_reap_kids();
         }
     }
-
-    dcc_parent_terminate();
 }
 
-
 /**
  * Called in the daemon parent when a connection is received.  Either
  * forks a new child, or handles it in the process (debug mode), as
@@ -295,33 +225,6 @@ static int dcc_serve_connection(int listen_fd, int acc_fd)
     return ret;
 }
 
-
-/**
- * Just log, remove pidfile, and exit.
- *
- * The children will close down in their own time when they've finished whatever they're doing.
- *
- * Called when the parent gets a signal.
- *
- * This used to also kill all daemon children, but that's not such a
- * good idea: if they're busy serving a connection, it really does no
- * harm to leave them running, and it is probably cleaner to let them
- * finish their work.  The main socket gets unbound, so no more
- * connections will be accepted.
- **/
-static void dcc_parent_terminate(void)
-{
-    /* This used to kill our process group, but that's wrong for
-     * --no-detach, and not helpful for shutting down children. */
-
-    dcc_remove_pid();
-
-    rs_log_notice("terminating on signal %d", parent_exit_signal);
-
-    dcc_exit(EXIT_SUCCESS);
-}
-
-
 /**
  * Fork a child to handle an incoming job.  Not called when in
  * opt_no_fork mode.
@@ -340,7 +243,7 @@ static int dcc_start_child(int listen_fd, int accepted_fd)
         /* doesn't return */
     } else if (kid == -1) {
         rs_log_error("fork failed: %s", strerror(errno));
-        return -1;
+        return EXIT_OUT_OF_MEMORY; /* probably */
     }
 
     return 0;
@@ -383,18 +286,18 @@ static void dcc_save_pid(pid_t pid)
 
 /**
  * Remove our pid file on exit.
+ *
+ * Must be reentrant -- called from signal handler.
  **/
-static void dcc_remove_pid(void)
+void dcc_remove_pid(void)
 {
     if (!arg_pid_file)
         return;
 
-#if 0
     if (unlink(arg_pid_file)) {
         rs_log_warning("failed to remove pid file %s: %s",
                        arg_pid_file, strerror(errno));
     }
-#endif
 }
 
 
@@ -443,4 +346,8 @@ static void dcc_detach(void)
         close(i); 
         open("/dev/null", O_RDWR);
     }
+
+    /* If there's a lifetime limit on this server (for testing) then it needs
+     * to apply after detaching as well. */
+    dcc_set_lifetime();
 }
