@@ -30,8 +30,9 @@ int dcc_spawn_child(char **argv, pid_t *pidptr,
                     const char *, const char *, const char *);
 
 int dcc_collect_child(pid_t pid, int *wait_status, long *, long *);
-int dcc_critique_status(int s, const char *, const char *hostname);
-void dcc_note_execution(const char *hostname, char **argv);
+int dcc_critique_status(int s, const char *, struct dcc_hostdef *host,
+                        int verbose);
+void dcc_note_execution(struct dcc_hostdef *host, char **argv);
 int dcc_report_rusage(const char *command,
                       long utime_usec,
                       long stime_usec);
@@ -43,4 +44,3 @@ void dcc_reset_signal(int whichsig);
 #  define W_EXITCODE(exit, signal) ((exit)<<8 | (signal))
 #endif 
 
-void dcc_recursion_safeguard(void);
