@@ -48,4 +48,6 @@ enum dcc_exitcode {
     EXIT_SETUID_FAILED            = 112, /**< Failed to discard privileges */
     EXIT_ACCESS_DENIED            = 113, /**< Network access denied */
     EXIT_BUSY                     = 114, /**< In use by another process. */
+    EXIT_NO_SUCH_FILE             = 115,
+    EXIT_NO_HOSTS                 = 116, 
 };
