@@ -1,7 +1,7 @@
 /* -*- c-file-style: "java"; indent-tabs-mode: nil -*-
  * 
  * distcc -- A simple distributed compiler system
- * $Header: /data/cvs/distcc/src/h_hosts.c,v 1.8 2003/02/25 00:27:25 mbp Exp $ 
+ * $Header: /data/cvs/distcc/src/h_hosts.c,v 1.11 2003/05/21 06:26:30 mbp Exp $ 
  *
  * Copyright (C) 2002 by Martin Pool <mbp@samba.org>
  *
@@ -44,7 +44,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <assert.h>
 #include <string.h>
 #include <errno.h>
 #include <time.h>
@@ -72,7 +71,7 @@ int main(int UNUSED(argc), char **argv)
         rs_trace_set_level(RS_LOG_DEBUG);
     }
     
-    if ((ret = dcc_parse_hosts_env(&list, &nhosts)) != 0) {
+    if ((ret = dcc_get_hostlist(&list, &nhosts)) != 0) {
         rs_log_error("failed to parse \"%s\"", getenv("DISTCC_HOSTS"));
         exit(ret);
     }
