@@ -8,12 +8,21 @@
 #endif
 
 
-#ifndef HAVE_VASPRINTF_DECL
+/* Note that the HAVE_DECL macros are defined to 0 if the declaration
+ * is not present, rather than being undefined as is the case for most
+ * autoconf tests. */
+
+
+#if !HAVE_DECL_VASPRINTF
 int vasprintf(char **ptr, const char *format, va_list ap);
 #endif
-#ifndef HAVE_SNPRINTF_DECL
+#if !HAVE_DECL_SNPRINTF
 int snprintf(char *,size_t ,const char *, ...) PRINTF_ATTRIBUTE(3,4);
 #endif
-#ifndef HAVE_ASPRINTF_DECL
+#if !HAVE_DECL_ASPRINTF
 int asprintf(char **,const char *, ...) PRINTF_ATTRIBUTE(2,3);
 #endif
+
+#if !HAVE_DECL_VSNPRINTF
+int vsnprintf(char *, size_t, const char *, va_list);
+#endif
