@@ -124,12 +124,7 @@ int dcc_strip_local_args(char **from, char ***out_argv)
     /* NULL-terminate */
     to[to_i] = NULL;
 
-    {
-        char *argstr;
-        argstr = dcc_argv_tostr(to);
-        rs_trace("result: %s", argstr);
-        free(argstr);
-    }
+    dcc_trace_argv("result", to);
 
     return 0;
 }
@@ -174,12 +169,7 @@ int dcc_strip_dasho(char **from, char ***out_argv)
     /* NULL-terminate */
     to[to_i] = NULL;
 
-    {
-        char *argstr;
-        argstr = dcc_argv_tostr(to);
-        rs_trace("result: %s", argstr);
-        free(argstr);
-    }
+    dcc_trace_argv("result", to);
 
     return 0;
 }
