@@ -1,9 +1,8 @@
 /*= -*- c-basic-offset: 4; indent-tabs-mode: nil; -*-
  *
- * librsync -- library for network deltas
- * $Id: trace.c,v 1.26 2003/02/25 00:27:25 mbp Exp $
+ * ecolog - Reusable application logging library.
  *
- * Copyright (C) 2000, 2001, 2002 by Martin Pool <mbp@samba.org>
+ * Copyright (C) 2000 - 2003 by Martin Pool <mbp@samba.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -28,15 +27,6 @@
                                       */
 
 
-
-/*
- * @todo Have a bit in the log level that says not to include the
- * function name.
- *
- * @todo Always show program/pid in log files.
- */
-
-
 #include "config.h"
 
 #include <unistd.h>
@@ -45,12 +35,11 @@
 #include <string.h>
 #include <errno.h>
 #include <stdlib.h>
-#include <assert.h>
 #include <stdarg.h>
 #include <syslog.h>
 
 #include "trace.h"
-
+#include "snprintf.h"
 
 struct rs_logger_list {
     rs_logger_fn               *fn;
