@@ -82,7 +82,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <assert.h>
 #include <string.h>
 #include <fcntl.h>
 #include <errno.h>
@@ -98,6 +97,8 @@
 #include "tempfile.h"
 #include "lock.h"
 #include "where.h"
+#include "exitcode.h"
+#include "backoff.h"
 
 
 static int dcc_lock_one(struct dcc_hostdef *hostlist,
@@ -106,35 +107,49 @@ static int dcc_lock_one(struct dcc_hostdef *hostlist,
                         int *cpu_lock_fd);
 
 
-int dcc_pick_host_from_env(struct dcc_hostdef **buildhost,
-                           int *xmit_lock_fd,
-                           int *cpu_lock_fd)
+int dcc_pick_host_from_list(struct dcc_hostdef **buildhost,
+                            int *xmit_lock_fd,
+                            int *cpu_lock_fd)
 {
     struct dcc_hostdef *hostlist;
     int ret;
     int n_hosts;
     
-    if ((ret = dcc_parse_hosts_env(&hostlist, &n_hosts)) != 0) {
-        /* an error occured; but let's be helpful and build locally
-         * rather than giving up. */
-        *buildhost = (struct dcc_hostdef *) dcc_hostdef_local;
-        return 0;
+    if ((ret = dcc_get_hostlist(&hostlist, &n_hosts)) != 0) {
+        return EXIT_NO_HOSTS;
     }
 
+    if ((ret = dcc_remove_disliked(&hostlist)))
+        return ret;
+
+    if (!hostlist) {
+        return EXIT_NO_HOSTS;
+    }
+    
     return dcc_lock_one(hostlist, buildhost, xmit_lock_fd, cpu_lock_fd);
 }
 
 
 static void dcc_lock_pause(void)
 {
-    /* Some people might want to randomize this, but I think the
-     * randomization introduced by scheduling and by tasks starting at
-     * different times is probably enough for now.
+    /* This could do with some tuning.
      *
      * My assumption basically is that polling a little too often is
-     * relatively cheap; sleeping when we should be working is bad. */
-    rs_trace("nothing available, sleeping...");
-    usleep(100000);         /* 0.1s, to start with */
+     * relatively cheap; sleeping when we should be working is bad.  However,
+     * if we hit this code at all we're overloaded, so sleeping a while is
+     * perhaps OK.
+     *
+     * We don't use exponential backoff, because that would tend to prefer
+     * later arrivals and penalize jobs that have been waiting for a long
+     * time.  This would mean more compiler processes hanging around than is
+     * really necessary, and also by making jobs complete very-out-of-order is
+     * more likely to find Makefile bugs. */
+
+    int pause_time = 2;
+    
+    rs_trace("nothing available, sleeping %ds...", pause_time);
+
+    sleep(pause_time);
 }
 
 
@@ -154,14 +169,17 @@ static int dcc_lock_one(struct dcc_hostdef *hostlist,
 {
     struct dcc_hostdef *h;
     int i_cpu;
+    int ret;
 
     while (1) {
         for (i_cpu = 0; i_cpu < 50; i_cpu++) {
             for (h = hostlist; h; h = h->next) {
                 if (i_cpu >= h->n_slots)
                     continue;
+
+                ret = dcc_lock_host("cpu", h, i_cpu, 0, cpu_lock_fd);
                 
-                if (dcc_lock_host("cpu", h, i_cpu, 0, cpu_lock_fd) == 0) {
+                if (ret == 0) {
                     /* If this is localhost, there is no transmission phase
                      * and we don't take a lock */
                     if (h->mode == DCC_MODE_LOCAL) {
@@ -177,6 +195,11 @@ static int dcc_lock_one(struct dcc_hostdef *hostlist,
                         /* release lock */
                         dcc_unlock(*cpu_lock_fd);
                     }
+                } else if (ret == EXIT_BUSY) {
+                    continue;
+                } else {
+                    rs_log_error("failed to lock");
+                    return ret;
                 }
             }
         }
