@@ -540,6 +540,12 @@ int main(void) {
         self.assert_equal(msgs, "hello world\n")
 
 
+class WriteDevNull_Case(CompileHello_Case):
+    def runtest(self):
+        self.compile()
+        
+    def compileCmd(self):
+        return "DISTCC_FALLBACK=0 distcc -c -o /dev/null -c %s" % (self.sourceFilename())
 
 
 class MultipleCompile_Case(Compilation_Case):
@@ -1045,6 +1051,7 @@ tests = [BadLogFile_Case,
          StripArgs_Case,
          StartStopDaemon_Case,
          CompileHello_Case,
+         WriteDevNull_Case,
          CppError_Case,
          BadInclude_Case,
          PreprocessPlainText_Case,
